"use strict";
const vue = require("@vue/language-core");
const vueTs = require("@vue/typescript");
const vscode_uri_1 = require("vscode-uri");
const fs = require("fs");
const init = (modules) => {
    const { typescript: ts } = modules;
    const externalFiles = new Map();
    const pluginModule = {
        create(info) {
            const projectName = info.project.getProjectName();
            if (!info.project.fileExists(projectName)) {
                // project name not a tsconfig path, this is a inferred project
                return info.languageService;
            }
            const parsed = vue.createParsedCommandLine(ts, ts.sys, projectName);
            const vueFileNames = parsed.fileNames.filter(fileName => fileName.endsWith('.vue'));
            if (!vueFileNames.length) {
                // no vue file
                return info.languageService;
            }
            externalFiles.set(info.project, vueFileNames);
            // fix: https://github.com/vuejs/language-tools/issues/205
            if (!info.project.__vue_getScriptKind) {
                info.project.__vue_getScriptKind = info.project.getScriptKind;
                info.project.getScriptKind = fileName => {
                    if (fileName.endsWith('.vue')) {
                        return ts.ScriptKind.Deferred;
                    }
                    return info.project.__vue_getScriptKind(fileName);
                };
            }
            const vueTsLsHost = {
                getCompilationSettings: () => info.project.getCompilationSettings(),
                getCurrentDirectory: () => info.project.getCurrentDirectory(),
                getProjectVersion: () => info.project.getProjectVersion(),
                getProjectReferences: () => info.project.getProjectReferences(),
                getScriptFileNames: () => [
                    ...info.project.getScriptFileNames(),
                    ...vueFileNames,
                ],
                getScriptSnapshot: (fileName) => info.project.getScriptSnapshot(fileName),
            };
            const uriToFileName = (uri) => vscode_uri_1.URI.parse(uri).fsPath.replace(/\\/g, '/');
            const fileNameToUri = (fileName) => vscode_uri_1.URI.file(fileName).toString();
            const vueTsLs = vueTs.createLanguageService(vueTsLsHost, parsed.vueOptions, ts, {
                uriToFileName,
                fileNameToUri,
                rootUri: vscode_uri_1.URI.parse(fileNameToUri(vueTsLsHost.getCurrentDirectory())),
                fs: {
                    stat(uri) {
                        if (uri.startsWith('file://')) {
                            try {
                                const stats = fs.statSync(uriToFileName(uri), { throwIfNoEntry: false });
                                if (stats) {
                                    return {
                                        type: stats.isFile() ? 1
                                            : stats.isDirectory() ? 2
                                                : stats.isSymbolicLink() ? 64
                                                    : 0,
                                        ctime: stats.ctimeMs,
                                        mtime: stats.mtimeMs,
                                        size: stats.size,
                                    };
                                }
                            }
                            catch { }
                        }
                    },
                    readFile(uri, encoding) {
                        if (uri.startsWith('file://')) {
                            return fs.readFileSync(uriToFileName(uri), { encoding: encoding ?? 'utf-8' });
                        }
                    },
                    readDirectory(uri) {
                        if (uri.startsWith('file://')) {
                            const dirName = uriToFileName(uri);
                            const files = fs.existsSync(dirName) ? fs.readdirSync(dirName, { withFileTypes: true }) : [];
                            return files.map(file => {
                                return [file.name, file.isFile() ? 1
                                        : file.isDirectory() ? 2
                                            : file.isSymbolicLink() ? 64
                                                : 0];
                            });
                        }
                        return [];
                    },
                },
            });
            return new Proxy(info.languageService, {
                get: (target, property) => {
                    if (property === 'getSemanticDiagnostics'
                        || property === 'getEncodedSemanticClassifications'
                        || property === 'getCompletionsAtPosition'
                        || property === 'getCompletionEntryDetails'
                        || property === 'getCompletionEntrySymbol'
                        || property === 'getQuickInfoAtPosition'
                        || property === 'getSignatureHelpItems'
                        || property === 'getRenameInfo'
                        || property === 'findRenameLocations'
                        || property === 'getDefinitionAtPosition'
                        || property === 'getDefinitionAndBoundSpan'
                        || property === 'getTypeDefinitionAtPosition'
                        || property === 'getImplementationAtPosition'
                        || property === 'getReferencesAtPosition'
                        || property === 'findReferences') {
                        return vueTsLs[property];
                    }
                    return target[property];
                },
            });
        },
        getExternalFiles(project) {
            return externalFiles.get(project) ?? [];
        },
    };
    return pluginModule;
};
module.exports = init;
//# sourceMappingURL=index.js.map